@@ -54,7 +54,7 @@ public:
     {
     }
 
-    ~ProxyModel()
+    ~ProxyModel() override
     {
         qDeleteAll(mFrontItems);
         qDeleteAll(mBackItems);
@@ -73,10 +73,21 @@ public:
         // As we display UID(0) this is ok. We probably need a get Best UID at some point.
         const auto lUid = leftKey.userID(0);
         const auto rUid = rightKey.userID(0);
+        if (lUid.isNull()) {
+            return false;
+        }
+        if (rUid.isNull()) {
+            return true;
+        }
+        int cmp = strcmp (lUid.id(), rUid.id());
+        if (cmp) {
+            return cmp < 0;
+        }
+
         if (lUid.validity() == rUid.validity()) {
             /* Both are the same check which one is newer. */
             time_t oldTime = 0;
-            for (const GpgME::Subkey s: leftKey.subkeys()) {
+            for (const GpgME::Subkey &s: leftKey.subkeys()) {
                 if (s.isRevoked() || s.isInvalid() || s.isDisabled()) {
                     continue;
                 }
@@ -85,7 +96,7 @@ public:
                 }
             }
             time_t newTime = 0;
-            for (const GpgME::Subkey s: rightKey.subkeys()) {
+            for (const GpgME::Subkey &s: rightKey.subkeys()) {
                 if (s.isRevoked() || s.isInvalid() || s.isDisabled()) {
                     continue;
                 }
@@ -147,8 +158,8 @@ public:
     {
         if (!isCustomItem(index.row())) {
             const int row = index.row() - mFrontItems.count();
-            const QModelIndex idx = createIndex(row, index.column(), index.internalPointer());
-            return QSortFilterProxyModel::mapToSource(idx);
+
+            return sourceModel()->index(row, index.column());
         } else {
             return QModelIndex();
         }
@@ -227,12 +238,20 @@ public:
                 name = dn[QStringLiteral("CN")];
                 email = dn[QStringLiteral("EMAIL")];
             }
-            return i18nc("Name <email> (validity, type, created: date)", "%1 (%2, %3, created: %4)",
+            return i18nc("Name <email> (validity, type, created: date)", "%1 (%2, %3 created: %4)",
                          email.isEmpty() ? name : name.isEmpty() ? email : i18nc("Name <email>", "%1 <%2>", name, email),
                          Kleo::Formatting::complianceStringShort(key),
-                         key.protocol() == GpgME::OpenPGP ? i18n("OpenPGP") : i18n("S/MIME"),
+                         Kleo::KeyCache::instance()->pgpOnly() ? QString() :
+                            key.protocol() == GpgME::OpenPGP ? i18n("OpenPGP") + QLatin1Char(',') : i18n("S/MIME") + QLatin1Char(','),
                          Kleo::Formatting::creationDateString(key));
         }
+        case Qt::ToolTipRole:
+            return Kleo::Formatting::toolTip(key, Kleo::Formatting::Validity |
+                                                  Kleo::Formatting::Issuer |
+                                                  Kleo::Formatting::Subject |
+                                                  Kleo::Formatting::Fingerprint |
+                                                  Kleo::Formatting::ExpiryDates |
+                                                  Kleo::Formatting::UserIDs);
         case Qt::DecorationRole:
             return Kleo::Formatting::iconForUid(key.userID(0));
         default:
@@ -259,6 +278,44 @@ public:
     {
     }
 
+    /* Selects the first key with a UID addrSpec that matches
+     * the mPerfectMatchMbox variable.
+     *
+     * The idea here is that if there are keys like:
+     *
+     * tom-store@abc.com
+     * susi-store@abc.com
+     * store@abc.com
+     *
+     * And the user wants to send a mail to "store@abc.com"
+     * the filter should still show tom and susi (because they
+     * both are part of store) but the key for "store" should
+     * be preselected.
+     *
+     * Returns true if one was selected. False otherwise. */
+    bool selectPerfectIdMatch() const
+    {
+        if (mPerfectMatchMbox.isEmpty()) {
+            return false;
+        }
+
+        for (int i = 0; i < proxyModel->rowCount(); ++i) {
+            const auto idx = proxyModel->index(i, 0, QModelIndex());
+            const auto key = proxyModel->data(idx, Kleo::KeyListModelInterface::KeyRole).value<GpgME::Key>();
+            if (key.isNull()) {
+                // WTF?
+                continue;
+            }
+            for (const auto &uid: key.userIDs()) {
+                if (QString::fromStdString(uid.addrSpec()) == mPerfectMatchMbox) {
+                    q->setCurrentIndex(i);
+                    return true;
+                }
+            }
+        }
+        return false;
+    }
+
     Kleo::AbstractKeyListModel *model = nullptr;
     Kleo::KeyListSortFilterProxyModel *sortFilterProxy = nullptr;
     ProxyModel *proxyModel = nullptr;
@@ -267,6 +324,7 @@ public:
     bool wasEnabled = false;
     bool useWasEnabled = false;
     bool secretOnly;
+    QString mPerfectMatchMbox;
 
 private:
     KeySelectionCombo * const q;
@@ -347,6 +405,10 @@ void KeySelectionCombo::init()
         d->model->useKeyCache(true, d->secretOnly);
         Q_EMIT keyListingFinished();
     }
+
+    connect(this, QOverload<int>::of(&QComboBox::currentIndexChanged), this, [this] () {
+            setToolTip(currentData(Qt::ToolTipRole).toString());
+        });
 }
 
 
@@ -365,6 +427,7 @@ std::shared_ptr<const KeyFilter> KeySelectionCombo::keyFilter() const
 void KeySelectionCombo::setIdFilter(const QString &id)
 {
     d->sortFilterProxy->setFilterRegExp(id);
+    d->mPerfectMatchMbox = id;
     setCurrentKey(d->defaultKey);
 }
 
@@ -383,7 +446,10 @@ void Kleo::KeySelectionCombo::setCurrentKey(const GpgME::Key &key)
     const int idx = findData(QVariant::fromValue(key), Kleo::KeyListModelInterface::KeyRole, Qt::MatchExactly);
     if (idx > -1) {
         setCurrentIndex(idx);
+    } else {
+        d->selectPerfectIdMatch();
     }
+    setToolTip(currentData(Qt::ToolTipRole).toString());
 }
 
 void Kleo::KeySelectionCombo::setCurrentKey(const QString &fingerprint)
@@ -393,10 +459,14 @@ void Kleo::KeySelectionCombo::setCurrentKey(const QString &fingerprint)
         const auto key = d->proxyModel->data(idx, Kleo::KeyListModelInterface::KeyRole).value<GpgME::Key>();
         if (!key.isNull() && fingerprint == QString::fromLatin1(key.primaryFingerprint())) {
             setCurrentIndex(i);
+            setToolTip(currentData(Qt::ToolTipRole).toString());
             return;
         }
     }
-    setCurrentIndex(0);
+    if (!d->selectPerfectIdMatch()) {
+        setCurrentIndex(0);
+    }
+    setToolTip(currentData(Qt::ToolTipRole).toString());
 }
 
 void KeySelectionCombo::refreshKeys()
